// Ohhara Tetris Version 1.1
// Programmed by ohhara@postech.ac.kr
// 1998/02/17 ~ 1998/02/21
// This application can be distributed freely .
// This application needs JDK 1.1.X . 

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

class Tetris extends Frame implements WindowListener,KeyListener,ActionListener,Runnable
{

	public final static int tetrisX=10;
	public final static int tetrisY=32;
	public final static int tetrisXLen=20;
	public final static int tetrisYLen=20;
	public final static Color backColor=Color.black;

	private final static int startFlag=-5;
	private final static int scoreFlag=-6;
	private final static int winFlag=-7;

	private TetrisCanvas tetrisCanvas;
	private TetrisCanvas tetrisRemoteCanvas;
	private Color[][] blocks;
	private Panel panel1;
	private Panel panel2;
	private Panel panel3;
	private Label scoreLabel;
	private Label enemyScoreLabel;
	private Button serverButton;
	private Button clientButton;
	private Button newGameButton;
	private Button pauseButton;
	private Button quitButton;
	private Button aboutButton;

	private Color[][] curBlock;
	private int curShape;
	private Color curColor;
	private int startX;
	private int startY;
	private int endX;
	private int endY;
	private int initDelay=500;
	private int delay;
	private int score=0;
	private int enemyScore=0;
	private Random r;
	private boolean pause=false;

	private ServerSocket server;
	private Socket client;
	private boolean useNetwork=false;
	private String serverName="localhost";
	private int port=3001;
	private DataInputStream dis;
	private DataOutputStream dos;
	private Color[][] remoteBlocks;

	private Thread tetrisThread;
	private Thread tetrisRemoteThread;
	private Thread tetrisRemoteInputThread;

	Tetris()
	{

		blocks=new Color[tetrisY][tetrisX];
		tetrisCanvas=new TetrisCanvas();
		panel1=new Panel();
		panel2=new Panel();
		panel3=new Panel();
		curBlock=new Color[tetrisY][tetrisX];
		r=new Random();
		tetrisThread=new Thread(this,"tetrisThread");
		tetrisRemoteThread=new Thread(this,"tetrisRemoteThread");
		tetrisRemoteInputThread=new Thread(this,"tetrisRemoteInputThread");

		for(int x=0;x<tetrisX;x++)
			for(int y=0;y<tetrisY;y++)
				blocks[y][x]=backColor;

		setLayout(new BorderLayout());
		setTitle("Ohhara Tetris 1.1");

		panel1.setLayout(new BorderLayout());
		panel1.add("North",tetrisCanvas);
		panel1.add("Center",scoreLabel=new Label("Your Score : "+score));
		add("West",panel1);

		panel2.setLayout(new GridLayout(20,1));
		panel2.add(newGameButton=new Button("New Game"));
		panel2.add(pauseButton=new Button("Pause"));
		panel2.add(serverButton=new Button("Server"));
		panel2.add(clientButton=new Button("Client"));
		panel2.add(quitButton=new Button("Quit"));
		panel2.add(aboutButton=new Button("About"));
		add("Center",panel2);

		panel3.setLayout(new BorderLayout());

		addWindowListener(this);
		addKeyListener(this);
		tetrisCanvas.addKeyListener(this);
		serverButton.addActionListener(this);
		clientButton.addActionListener(this);
		newGameButton.addActionListener(this);
		pauseButton.addActionListener(this);
		quitButton.addActionListener(this);
		aboutButton.addActionListener(this);

		pack();
		show();

	}

	public void setServerName(String serverName)
	{
		this.serverName=serverName;
	}
	public void setPort(int port)
	{
		this.port=port;
	}
	public void setUseNetwork(boolean useNetwork)
	{
		this.useNetwork=useNetwork;
	}

	public synchronized void createBlock()
	{
		float tmpShape;
		float tmpColor;
		float tmpFloat;

		for(int x=0;x<tetrisX;x++)
			for(int y=0;y<tetrisY;y++)
				curBlock[y][x]=backColor;

		tmpShape=r.nextFloat();
		tmpFloat=(float)(1.0/7.0);

		if(0.0<=tmpShape&&tmpShape<tmpFloat)
			curShape=0;
		else if(tmpFloat<=tmpShape&&tmpShape<tmpFloat*2)
			curShape=10;
		else if(tmpFloat*2<=tmpShape&&tmpShape<tmpFloat*3)
			curShape=20;
		else if(tmpFloat*3<=tmpShape&&tmpShape<tmpFloat*4)
			curShape=30;
		else if(tmpFloat*4<=tmpShape&&tmpShape<tmpFloat*5)
			curShape=40;
		else if(tmpFloat*5<=tmpShape&&tmpShape<tmpFloat*6)
			curShape=50;
		else if(tmpFloat*6<=tmpShape&&tmpShape<=tmpFloat*7)
			curShape=60;

		tmpColor=tmpShape;

		if(0.0<=tmpColor&&tmpColor<tmpFloat)
			curColor=Color.blue;
		else if(tmpFloat<=tmpColor&&tmpColor<tmpFloat*2)
			curColor=Color.cyan;
		else if(tmpFloat*2<=tmpColor&&tmpColor<tmpFloat*3)
			curColor=Color.orange;
		else if(tmpFloat*3<=tmpColor&&tmpColor<tmpFloat*4)
			curColor=Color.red;
		else if(tmpFloat*4<=tmpColor&&tmpColor<tmpFloat*5)
			curColor=Color.yellow;
		else if(tmpFloat*5<=tmpColor&&tmpColor<tmpFloat*6)
			curColor=Color.pink;
		else if(tmpFloat*6<=tmpColor&&tmpColor<=tmpFloat*7)
			curColor=Color.green;

		switch(curShape)
		{
			case 0:
				startX=4;
				startY=0;
				endX=5;
				endY=1;
				blocks[0][4]=curBlock[0][4]=curColor;
				blocks[0][5]=curBlock[0][5]=curColor;
				blocks[1][4]=curBlock[1][4]=curColor;
				blocks[1][5]=curBlock[1][5]=curColor;
				break;
			case 10:
				startX=3;
				startY=1;
				endX=6;
				endY=1;
				blocks[1][3]=curBlock[1][3]=curColor;
				blocks[1][4]=curBlock[1][4]=curColor;
				blocks[1][5]=curBlock[1][5]=curColor;
				blocks[1][6]=curBlock[1][6]=curColor;
				break;
			case 20:
				startX=3;
				startY=0;
				endX=5;
				endY=1;
				blocks[0][4]=curBlock[0][4]=curColor;
				blocks[1][3]=curBlock[1][3]=curColor;
				blocks[1][4]=curBlock[1][4]=curColor;
				blocks[1][5]=curBlock[1][5]=curColor;
				break;
			case 30:
				startX=3;
				startY=0;
				endX=5;
				endY=1;
				blocks[0][3]=curBlock[0][3]=curColor;
				blocks[1][3]=curBlock[1][3]=curColor;
				blocks[1][4]=curBlock[1][4]=curColor;
				blocks[1][5]=curBlock[1][5]=curColor;
				break;
			case 40:
				startX=3;
				startY=0;
				endX=5;
				endY=1;
				blocks[0][5]=curBlock[0][5]=curColor;
				blocks[1][3]=curBlock[1][3]=curColor;
				blocks[1][4]=curBlock[1][4]=curColor;
				blocks[1][5]=curBlock[1][5]=curColor;
				break;
			case 50:
				startX=3;
				startY=0;
				endX=5;
				endY=1;
				blocks[0][3]=curBlock[0][3]=curColor;
				blocks[0][4]=curBlock[0][4]=curColor;
				blocks[1][4]=curBlock[1][4]=curColor;
				blocks[1][5]=curBlock[1][5]=curColor;
				break;
			case 60:
				startX=3;
				startY=0;
				endX=5;
				endY=1;
				blocks[0][4]=curBlock[0][4]=curColor;
				blocks[0][5]=curBlock[0][5]=curColor;
				blocks[1][3]=curBlock[1][3]=curColor;
				blocks[1][4]=curBlock[1][4]=curColor;
				break;
			default:
				break;
		}
		tetrisCanvas.updateBlocks(blocks);
	}

	public synchronized void moveLeft()
	{
		if(pause)
			return;
		for(int y=startY;y<=endY;y++)
			for(int x=startX;x<=endX;x++)
				if(!(curBlock[y][x].equals(backColor)))
					if(x==0)
						return;
					else if(!(blocks[y][x-1].equals(backColor)))
						if(curBlock[y][x-1].equals(backColor))
							return;
		for(int y=startY;y<=endY;y++)
			for(int x=startX;x<=endX;x++)
				if(!(curBlock[y][x].equals(backColor)))
				{
					curBlock[y][x-1]=curBlock[y][x];
					curBlock[y][x]=backColor;
					blocks[y][x-1]=blocks[y][x];
					blocks[y][x]=backColor;
				}
		startX--;
		endX--;
		tetrisCanvas.updateBlocks(blocks);
	}
	public synchronized void moveRight()
	{
		if(pause)
			return;
		for(int y=startY;y<=endY;y++)
			for(int x=endX;x>=startX;x--)
				if(!(curBlock[y][x].equals(backColor)))
					if(x==tetrisX-1)
						return;
					else if(!(blocks[y][x+1].equals(backColor)))
						if(curBlock[y][x+1].equals(backColor))
							return;
		for(int y=startY;y<=endY;y++)
			for(int x=endX;x>=startX;x--)
				if(!(curBlock[y][x].equals(backColor)))
				{
					curBlock[y][x+1]=curBlock[y][x];
					curBlock[y][x]=backColor;
					blocks[y][x+1]=blocks[y][x];
					blocks[y][x]=backColor;
				}
		startX++;
		endX++;
		tetrisCanvas.updateBlocks(blocks);
	}
	public synchronized void rotate()
	{
		if(pause)
			return;
		try
		{
			switch(curShape)
			{
				case 0:
					break;
				case 10:
					if(blocks[startY-2][startX+1].equals(backColor)
					&&blocks[startY-1][startX+1].equals(backColor)
					&&blocks[startY+1][startX+1].equals(backColor))
					{
						curBlock[startY-2][startX+1]=blocks[startY-2][startX+1]=curColor;
						curBlock[startY-1][startX+1]=blocks[startY-1][startX+1]=curColor;
						curBlock[startY+1][startX+1]=blocks[startY+1][startX+1]=curColor;
						curBlock[startY][startX]=blocks[startY][startX]=backColor;
						curBlock[startY][startX+2]=blocks[startY][startX+2]=backColor;
						curBlock[startY][startX+3]=blocks[startY][startX+3]=backColor;
						startX++;
						startY-=2;
						endX-=2;
						endY++;
						curShape=11;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 11:
					if(blocks[startY+2][startX-1].equals(backColor)
					&&blocks[startY+2][startX+1].equals(backColor)
					&&blocks[startY+2][startX+2].equals(backColor))
					{
						curBlock[startY+2][startX-1]=blocks[startY+2][startX-1]=curColor;
						curBlock[startY+2][startX+1]=blocks[startY+2][startX+1]=curColor;
						curBlock[startY+2][startX+2]=blocks[startY+2][startX+2]=curColor;
						curBlock[startY][startX]=blocks[startY][startX]=backColor;
						curBlock[startY+1][startX]=blocks[startY+1][startX]=backColor;
						curBlock[startY+3][startX]=blocks[startY+3][startX]=backColor;
						startX--;
						startY+=2;
						endX+=2;
						endY--;
						curShape=10;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 20:
					if(blocks[startY+2][startX+1].equals(backColor))
					{
						curBlock[startY+2][startX+1]=blocks[startY+2][startX+1]=curColor;
						curBlock[startY+1][startX+2]=blocks[startY+1][startX+2]=backColor;
						endX--;
						endY++;
						curShape=21;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 21:
					if(blocks[startY+1][startX+2].equals(backColor))
					{
						curBlock[startY+1][startX+2]=blocks[startY+1][startX+2]=curColor;
						curBlock[startY][startX+1]=blocks[startY][startX+1]=backColor;
						startY++;
						endX++;
						curShape=22;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 22:
					if(blocks[startY-1][startX+1].equals(backColor))
					{
						curBlock[startY-1][startX+1]=blocks[startY-1][startX+1]=curColor;
						curBlock[startY][startX]=blocks[startY][startX]=backColor;
						startX++;
						startY--;
						curShape=23;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 23:
					if(blocks[startY+1][startX-1].equals(backColor))
					{
						curBlock[startY+1][startX-1]=blocks[startY+1][startX-1]=curColor;
						curBlock[startY+2][startX]=blocks[startY+2][startX]=backColor;
						startX--;
						endY--;
						curShape=20;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 30:
					if(blocks[startY][startX+2].equals(backColor)
					&&blocks[startY-1][startX+2].equals(backColor))
					{
						curBlock[startY][startX+2]=blocks[startY][startX+2]=curColor;
						curBlock[startY-1][startX+2]=blocks[startY-1][startX+2]=curColor;
						curBlock[startY][startX]=blocks[startY][startX]=backColor;
						curBlock[startY+1][startX]=blocks[startY+1][startX]=backColor;
						startX++;
						startY--;
						curShape=31;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 31:
					if(blocks[startY][startX-1].equals(backColor)
					&&blocks[startY][startX].equals(backColor))
					{
						curBlock[startY][startX-1]=blocks[startY][startX-1]=curColor;
						curBlock[startY][startX]=blocks[startY][startX]=curColor;
						curBlock[startY+2][startX]=blocks[startY+2][startX]=backColor;
						curBlock[startY+2][startX+1]=blocks[startY+2][startX+1]=backColor;
						startX--;
						endY--;
						curShape=32;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 32:
					if(blocks[startY+1][startX].equals(backColor)
					&&blocks[startY+2][startX].equals(backColor))
					{
						curBlock[startY+1][startX]=blocks[startY+1][startX]=curColor;
						curBlock[startY+2][startX]=blocks[startY+2][startX]=curColor;
						curBlock[startY][startX+2]=blocks[startY][startX+2]=backColor;
						curBlock[startY+1][startX+2]=blocks[startY+1][startX+2]=backColor;
						endX--;
						endY++;
						curShape=33;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 33:
					if(blocks[startY+2][startX+1].equals(backColor)
					&&blocks[startY+2][startX+2].equals(backColor))
					{
						curBlock[startY+2][startX+1]=blocks[startY+2][startX+1]=curColor;
						curBlock[startY+2][startX+2]=blocks[startY+2][startX+2]=curColor;
						curBlock[startY][startX]=blocks[startY][startX]=backColor;
						curBlock[startY][startX+1]=blocks[startY][startX+1]=backColor;
						startY++;
						endX++;
						curShape=30;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 40:
					if(blocks[startY-1][startX+1].equals(backColor)
					&&blocks[startY-1][startX+2].equals(backColor))
					{
						curBlock[startY-1][startX+1]=blocks[startY-1][startX+1]=curColor;
						curBlock[startY-1][startX+2]=blocks[startY-1][startX+2]=curColor;
						curBlock[startY+1][startX]=blocks[startY+1][startX]=backColor;
						curBlock[startY+1][startX+1]=blocks[startY+1][startX+1]=backColor;
						startX++;
						startY--;
						curShape=41;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 41:
					if(blocks[startY][startX-1].equals(backColor)
					&&blocks[startY+1][startX-1].equals(backColor))
					{
						curBlock[startY][startX-1]=blocks[startY][startX-1]=curColor;
						curBlock[startY+1][startX-1]=blocks[startY+1][startX-1]=curColor;
						curBlock[startY+1][startX+1]=blocks[startY+1][startX+1]=backColor;
						curBlock[startY+2][startX+1]=blocks[startY+2][startX+1]=backColor;
						startX--;
						endY--;
						curShape=42;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 42:
					if(blocks[startY+2][startX].equals(backColor)
					&&blocks[startY+2][startX+1].equals(backColor))
					{
						curBlock[startY+2][startX]=blocks[startY+2][startX]=curColor;
						curBlock[startY+2][startX+1]=blocks[startY+2][startX+1]=curColor;
						curBlock[startY][startX+1]=blocks[startY][startX+1]=backColor;
						curBlock[startY][startX+2]=blocks[startY][startX+2]=backColor;
						endX--;
						endY++;
						curShape=43;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 43:
					if(blocks[startY+1][startX+2].equals(backColor)
					&&blocks[startY+2][startX+2].equals(backColor))
					{
						curBlock[startY+1][startX+2]=blocks[startY+1][startX+2]=curColor;
						curBlock[startY+2][startX+2]=blocks[startY+2][startX+2]=curColor;
						curBlock[startY][startX]=blocks[startY][startX]=backColor;
						curBlock[startY+1][startX]=blocks[startY+1][startX]=backColor;
						startY++;
						endX++;
						curShape=40;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 50:
					if(blocks[startY+1][startX].equals(backColor)
					&&blocks[startY+2][startX].equals(backColor))
					{
						curBlock[startY+1][startX]=blocks[startY+1][startX]=curColor;
						curBlock[startY+2][startX]=blocks[startY+2][startX]=curColor;
						curBlock[startY][startX]=blocks[startY][startX]=backColor;
						curBlock[startY+1][startX+2]=blocks[startY+1][startX+2]=backColor;
						endX--;
						endY++;
						curShape=51;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 51:
					if(blocks[startY][startX].equals(backColor)
					&&blocks[startY+1][startX+2].equals(backColor))
					{
						curBlock[startY][startX]=blocks[startY][startX]=curColor;
						curBlock[startY+1][startX+2]=blocks[startY+1][startX+2]=curColor;
						curBlock[startY+1][startX]=blocks[startY+1][startX]=backColor;
						curBlock[startY+2][startX]=blocks[startY+2][startX]=backColor;
						endX++;
						endY--;
						curShape=50;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 60:
					if(blocks[startY][startX].equals(backColor)
					&&blocks[startY+2][startX+1].equals(backColor))
					{
						curBlock[startY][startX]=blocks[startY][startX]=curColor;
						curBlock[startY+2][startX+1]=blocks[startY+2][startX+1]=curColor;
						curBlock[startY][startX+1]=blocks[startY][startX+1]=backColor;
						curBlock[startY][startX+2]=blocks[startY][startX+2]=backColor;
						endX--;
						endY++;
						curShape=61;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				case 61:
					if(blocks[startY][startX+1].equals(backColor)
					&&blocks[startY][startX+2].equals(backColor))
					{
						curBlock[startY][startX+1]=blocks[startY][startX+1]=curColor;
						curBlock[startY][startX+2]=blocks[startY][startX+2]=curColor;
						curBlock[startY][startX]=blocks[startY][startX]=backColor;
						curBlock[startY+2][startX+1]=blocks[startY+2][startX+1]=backColor;
						endX++;
						endY--;
						curShape=60;
						tetrisCanvas.updateBlocks(blocks);
					}
					break;
				default:
					break;
			}
		}
		catch(IndexOutOfBoundsException ex)
		{
		}
	}
	public synchronized void moveDown()
	{
		if(pause)
			return;
		for(int y=endY;y>=startY;y--)
			for(int x=startX;x<=endX;x++)
				if(!(curBlock[y][x].equals(backColor)))
					if(y==tetrisY-1)
					{
						stopBlock();
						return;
					}
					else if(!(blocks[y+1][x].equals(backColor)))
						if(curBlock[y+1][x].equals(backColor))
						{
							stopBlock();
							return;
						}
		for(int y=endY;y>=startY;y--)
			for(int x=startX;x<=endX;x++)
				if(!(curBlock[y][x].equals(backColor)))
				{
					curBlock[y+1][x]=curBlock[y][x];
					curBlock[y][x]=backColor;
					blocks[y+1][x]=blocks[y][x];
					blocks[y][x]=backColor;
				}
		startY++;
		endY++;
		tetrisCanvas.updateBlocks(blocks);
	}
	public synchronized void fastMoveDown()
	{
		if(pause)
			return;
		while(true)
		{
			for(int y=endY;y>=startY;y--)
				for(int x=startX;x<=endX;x++)
					if(!(curBlock[y][x].equals(backColor)))
						if(y==tetrisY-1)
						{
							stopBlock();
							return;
						}
						else if(!(blocks[y+1][x].equals(backColor)))
							if(curBlock[y+1][x].equals(backColor))
							{
								stopBlock();
								return;
							}
			for(int y=endY;y>=startY;y--)
				for(int x=startX;x<=endX;x++)
					if(!(curBlock[y][x].equals(backColor)))
					{
						curBlock[y+1][x]=curBlock[y][x];
						curBlock[y][x]=backColor;
						blocks[y+1][x]=blocks[y][x];
						blocks[y][x]=backColor;
					}
			startY++;
			endY++;
			tetrisCanvas.updateBlocks(blocks);
		}
	}
	public synchronized void stopBlock()
	{
		for(int y=startY;y<=endY;y++)
			for(int x=startX;x<=endX;x++)
				if(!(curBlock[y][x].equals(backColor)))
					blocks[y][x]=curBlock[y][x];
		for(int y=startY;y<=endY;y++)
			for(int x=0;x<tetrisX;x++)
				if(blocks[y][x].equals(backColor))
					break;
				else if(x==tetrisX-1)
				{
					score++;
					scoreLabel.setText("Your Score : "+Integer.toString(score));
					if(useNetwork)
					{
						synchronized(dos)
						{
							try
							{
								dos.writeInt(scoreFlag);
								dos.writeInt(score);
								for(int atk=0;atk<tetrisX;atk++)
									if(!(curBlock[y][atk].equals(backColor)))
										dos.writeInt(backColor.getRGB());
									else
										dos.writeInt(blocks[y][atk].getRGB());
							}
							catch(IOException ex)
							{
							}
						}
					}
					for(int i=0;i<tetrisX;i++)
						for(int j=y;j>0;j--)
							blocks[j][i]=blocks[j-1][i];
				}
		if(endY<2)
		{
			if(useNetwork)
			{
				synchronized(dos)
				{
					try
					{
						dos.writeInt(winFlag);
					}
					catch(IOException ex)
					{
					}
				}
				new MessageDialog(this,"You lose");
				useNetwork=false;
				quitGame();
			}
		}
		else
			createBlock();
	}

	public void windowClosed(WindowEvent evt)
	{
	}
	public void windowDeiconified(WindowEvent evt)
	{
	}
	public void windowIconified(WindowEvent evt)
	{
	}
	public void windowActivated(WindowEvent evt)
	{
	}
	public void windowDeactivated(WindowEvent evt)
	{
	}
	public void windowOpened(WindowEvent evt)
	{
	}
	public void windowClosing(WindowEvent evt)
	{
		quitGame();
	}

	public void keyTyped(KeyEvent evt)
	{
	}
	public void keyPressed(KeyEvent evt)
	{
		switch(evt.getKeyCode())
		{
			case KeyEvent.VK_LEFT:
			case KeyEvent.VK_F:
				moveLeft();
				break;
			case KeyEvent.VK_RIGHT:
			case KeyEvent.VK_H:
				moveRight();
				break;
			case KeyEvent.VK_UP:
			case KeyEvent.VK_T:
				rotate();
				break;
			case KeyEvent.VK_DOWN:
			case KeyEvent.VK_B:
				moveDown();
				break;
			case KeyEvent.VK_SPACE:
				fastMoveDown();
				break;
			case KeyEvent.VK_ESCAPE:
			case KeyEvent.VK_Q:
				quitGame();
				break;
			default:
				break;
		}
	}
	public void keyReleased(KeyEvent evt)
	{
	}

	public void actionPerformed(ActionEvent evt)
	{
		if(evt.getActionCommand().equals("Server"))
			try
			{
				tetrisThread.suspend();
				pause=true;
				new ServerDialog(this);
				if(!useNetwork)
				{
					new MessageDialog(this,"Can't make server socket");
					tetrisThread.resume();
					pause=false;
					return;
				}
				server=new ServerSocket(port);
				client=server.accept();
				dos=new DataOutputStream(new BufferedOutputStream(client.getOutputStream()));
				dis=new DataInputStream(new BufferedInputStream(client.getInputStream()));
				tetrisRemoteCanvas=new TetrisCanvas();
				panel3.add("North",tetrisRemoteCanvas);
				panel3.add("Center",enemyScoreLabel=new Label("Enemy's Score : "+enemyScore));
				panel3.validate();
				add("East",panel3);
				validate();
				pack();
				newGame();
				remoteBlocks=new Color[tetrisY][tetrisX];
				if(tetrisRemoteThread.isAlive())
					tetrisRemoteThread.resume();
				else
					tetrisRemoteThread.start();
				if(tetrisRemoteInputThread.isAlive())
					tetrisRemoteInputThread.resume();
				else
					tetrisRemoteInputThread.start();
				serverButton.setEnabled(false);
				clientButton.setEnabled(false);
				newGameButton.setEnabled(false);
				pauseButton.setEnabled(false);
			}
			catch(IOException ex)
			{
				new MessageDialog(this,"Can't make server socket");
			}
		else if(evt.getActionCommand().equals("Client"))
			try
			{
				tetrisThread.suspend();
				pause=true;
				new ClientDialog(this);
				if(!useNetwork)
				{
					new MessageDialog(this,"Can't connect to server");
					tetrisThread.resume();
					pause=false;
					return;
				}
				client=new Socket(serverName,port);
				dos=new DataOutputStream(new BufferedOutputStream(client.getOutputStream()));
				dis=new DataInputStream(new BufferedInputStream(client.getInputStream()));
				tetrisRemoteCanvas=new TetrisCanvas();
				panel3.add("North",tetrisRemoteCanvas);
				panel3.add("Center",enemyScoreLabel=new Label("Enemy's Score : "+enemyScore));
				panel3.validate();
				add("East",panel3);
				validate();
				pack();

				newGame();
				remoteBlocks=new Color[tetrisY][tetrisX];
				if(tetrisRemoteThread.isAlive())
					tetrisRemoteThread.resume();
				else
					tetrisRemoteThread.start();
				if(tetrisRemoteInputThread.isAlive())
					tetrisRemoteInputThread.resume();
				else
					tetrisRemoteInputThread.start();
				serverButton.setEnabled(false);
				clientButton.setEnabled(false);
				newGameButton.setEnabled(false);
				pauseButton.setEnabled(false);
			}
			catch(IOException ex)
			{
				new MessageDialog(this,"Can't connect to server");
			}
		else if(evt.getActionCommand().equals("New Game"))
		{
			newGame();
		}
		else if(evt.getActionCommand().equals("Pause"))
		{
			tetrisThread.suspend();
			pauseButton.setLabel("Resume");
			pause=true;
		}
		else if(evt.getActionCommand().equals("Resume"))
		{
			tetrisThread.resume();
			pauseButton.setLabel("Pause");
			pause=false;
		}
		else if(evt.getActionCommand().equals("Quit"))
			quitGame();
		else if(evt.getActionCommand().equals("About"))
			new AboutDialog(this);
	}

	public void newGame()
	{
		if(!tetrisThread.isAlive())
			tetrisThread.start();
		tetrisThread.resume();
		for(int x=0;x<tetrisX;x++)
			for(int y=0;y<tetrisY;y++)
				blocks[y][x]=backColor;
		score=0;
		pause=false;
		scoreLabel.setText("Your Score : "+score);
		pauseButton.setLabel("Pause");
		createBlock();
	}
	public void quitGame()
	{
		if(useNetwork)
		{
			synchronized(dos)
			{
				try
				{
					dos.writeInt(winFlag);
				}
				catch(IOException ex)
				{
				}
			}
			tetrisThread.suspend();
			new MessageDialog(this,"You lose");
		}

		try
		{
			client.close();
			client=null;
		}
		catch(IOException ex)
		{
		}
		catch(NullPointerException ex)
		{
		}

		try
		{
			server.close();
			server=null;
		}
		catch(IOException ex)
		{
		}
		catch(NullPointerException ex)
		{
		}

		System.exit(0);
	}

	public void run()
	{
		try
		{
			if(Thread.currentThread().getName().equals("tetrisThread"))
				while(true)
				{
					delay=initDelay-score*10;
					if(delay<10)
						delay=10;
					Thread.sleep(delay);
					moveDown();
					requestFocus();
				}
			else if(Thread.currentThread().getName().equals("tetrisRemoteThread"))
				while(true)
				{
					delay=initDelay-score*10;
					if(delay<10)
						delay=10;
					Thread.sleep(delay);
					synchronized(dos)
					{
						dos.writeInt(startFlag);
						for(int x=0;x<tetrisX;x++)
							for(int y=0;y<tetrisY;y++)
								dos.writeInt(blocks[y][x].getRGB());
					}
				}
			else if(Thread.currentThread().getName().equals("tetrisRemoteInputThread"))
				while(true)
				{
					int flag=dis.readInt();
					if(flag==startFlag)
					{
						for(int x=0;x<Tetris.tetrisX;x++)
							for(int y=0;y<Tetris.tetrisY;y++)
								remoteBlocks[y][x]=new Color(dis.readInt());
						tetrisRemoteCanvas.updateBlocks(remoteBlocks);
					}
					else if(flag==scoreFlag)
					{
						enemyScore=dis.readInt();
						enemyScoreLabel.setText("Enemy's Score :"+enemyScore);
						synchronized(blocks)
						{
							for(int x=0;x<tetrisX;x++)
								for(int y=0;y<tetrisY;y++)
									if(!(curBlock[y][x].equals(backColor)))
										blocks[y][x]=curBlock[y][x]=backColor;
							for(int x=0;x<tetrisX;x++)
								for(int y=3;y<tetrisY;y++)
									blocks[y-1][x]=blocks[y][x];
							for(int x=0;x<tetrisX;x++)
								blocks[tetrisY-1][x]=new Color(dis.readInt());
							createBlock();
						}
					}
					else if(flag==winFlag)
					{
						tetrisThread.suspend();
						pause=true;
						new MessageDialog(this,"You win");
						useNetwork=false;
						quitGame();
					}
				}
		}
		catch(InterruptedException ex)
		{
		}
		catch(IOException ex)
		{
		}
	}

	public static void main(String[] args)
	{
		new Tetris();
	}

}

